package laboratoire4.strategies;

import laboratoire4.Action;
import laboratoire4.IPawn;
import laboratoire4.game.Game;
import laboratoire4.game.GameUtils;
import laboratoire4.pawns.PawnMovement;
import laboratoire4.pawns.PawnUtils;
import laboratoire4.pawns.SimulatedPawn;

import java.util.Collection;

public class AttackStrategy extends MiniMaxStrategy {
    private static final int ATTACK_DISTANCE_FROM_GOAL = 4;

    private long originalMinPushedCount;
    private long originalMinPusherCount;
    private long originalMaxPusherCount;

    protected AttackStrategy(Game game) {
        super(game);
    }

    @Override
    public EvaluationResult getNextMove() {
        originalMinPushedCount = GameUtils.getMinPawnsAsStream(game.getBoard(), game.getPlayer()).filter(p -> !p.isPusher()).count();
        originalMinPusherCount = GameUtils.getMinPawnsAsStream(game.getBoard(), game.getPlayer()).filter(IPawn::isPusher).count();
        originalMaxPusherCount = GameUtils.getMaxPawnsAsStream(game.getBoard(), game.getPlayer())
                .filter(IPawn::isPusher)
                .count();

        return super.getNextMove();
    }

    @Override
    protected int evaluateSimulation() {
        int score = 0;

        if (getMinPawns().stream().anyMatch(p -> p.getRow() == p.getPlayer().getGoal())) {
            return Integer.MIN_VALUE;
        }

        // region Pions max perdus
        Collection<SimulatedPawn> maxPawns = getMaxPawns();
        long pusherCount = maxPawns.stream().filter(IPawn::isPusher).count();
        score -= Math.pow((originalMaxPusherCount - pusherCount) * 10, 2);
        // endregion

        // region Pions min capturés
        long minPusherCount = getMinPawns().stream().filter(IPawn::isPusher).count();
        long minPushedCount = getMinPawns().stream().filter(p -> !p.isPusher()).count();
        score += (originalMinPushedCount - minPushedCount) * 5;
        score += (originalMinPusherCount - minPusherCount) * 10;
        // endregion

        for (SimulatedPawn pawn : maxPawns) {
            int distanceFromGoal = PawnUtils.distanceFromGoal(pawn);
            if (distanceFromGoal <= ATTACK_DISTANCE_FROM_GOAL) {
                score += ATTACK_DISTANCE_FROM_GOAL - distanceFromGoal + 1;

                if (PawnUtils.hasEasyWinPath(game.getBoard(), pawn)) {
                    score += Integer.MAX_VALUE / distanceFromGoal;
                }
            }

            int row = pawn.getRow();
            int col = pawn.getCol();

            if (row > 0 && row < 7) {
                IPawn facingPawn = game.getBoard()[row + pawn.getDirection()][col];
                if (facingPawn != null && !PawnUtils.areSamePlayers(pawn, facingPawn)) {
                    // On bloque potentiellement un pion adverse, qui ne peut pas nous attaquer
                    score += 10;
                }
            }

//            if (col > 0) {
//                IPawn leftPawn = game.getBoard()[row][col - 1];
//                if (leftPawn != null && !PawnUtils.areSamePlayers(pawn, leftPawn)) {
//                    score += 5;
//                }
//            }
//            if (col < 7) {
//                IPawn rightPawn = game.getBoard()[row][col + 1];
//                if (rightPawn != null && !PawnUtils.areSamePlayers(pawn, rightPawn)) {
//                    score += 5;
//                }
//            }
        }

        return score;
    }

    @Override
    public int getWeight() {
        Collection<IPawn> maxPawns = GameUtils.getMaxPawns(game.getBoard(), game.getPlayer());

        int maxWeight = 0;
        for (IPawn pawn : maxPawns) {
            int weight = 0;

            int distanceFromGoal = PawnUtils.distanceFromGoal(pawn);
            if (distanceFromGoal <= 1) {
                return WEIGHT_MAX;
            }

            if (distanceFromGoal == 2) {
                if (PawnUtils.hasEasyWinPath(game.getBoard(), pawn)) {
                    weight = 8;
                } else {
                    weight = 5;
                }
            }

            if (distanceFromGoal > 2 && distanceFromGoal <= ATTACK_DISTANCE_FROM_GOAL) {
                weight = 2;
            }

            if (weight > maxWeight) {
                maxWeight = weight;
            }
        }

        return maxWeight;
    }

    private boolean cantBeCaptured(Action<IPawn> action) {
        IPawn pawn = action.getPawn();
        PawnMovement movement = action.getMovement();

        int row = pawn.getRow() + pawn.getDirection();
        int col = pawn.getCol() + movement.getMove();

        return !PawnUtils.canBeCaptured(game, row, col, pawn.getPlayer());
    }
}
